char rcsid_bf_move[] = "$Id: bf_move.c,v 1.3 1992/08/10 17:28:14 pjames Exp $";

#include "my-stdio.h"

#include "execute.h"
#include "functions.h"
#include "list.h"
#include "parse_command.h"
#include "server.h"
#include "storage.h"
#include "utils.h"

struct bf_move_data {
  Objid what, where;
};

static Var
make_arglist(Objid what)
{
    Var		r;

    r = new_list(1);
    r.v.list[1].type = TYPE_OBJ;
    r.v.list[1].v.obj = what;

    return r;
}

static package
do_move(Var arglist, Byte next, struct bf_move_data *data, Objid progr)
{
    Objid		what = data->what, where = data->where;
    Objid		oid, oldloc;
    int			accepts;
    Var			args;
    enum error		e;
    
    switch (next) {
      case 1:			/* Check validity and decide `accepts' */
	if (!retrieve(what)  || (!retrieve(where)  &&  where != NOTHING))
	    return make_error_pack(E_INVARG);
	else if (retrieve(what)->owner != progr  &&  !is_wizard(progr))
	    return make_error_pack(E_PERM);
	else if (where == NOTHING)
	    accepts = 1;
	else {
	    args = make_arglist(what);
	    e = call_verb(where, "accept", args, 0);
	    /* e will not be E_INVIND or E_TYPE */
	    
	    if (e == E_NONE) 
		return make_call_pack(2, data);
	    else {
		free_var(args);
		if (e == E_MAXREC)
		    return make_error_pack(e);
		else /* (e == E_VERBNF) */
		    accepts = 0;
	    }
	}
	goto accepts_decided;

      case 2:			/* Returned from `accepts' call */
	accepts = is_true(arglist);

      accepts_decided:
	if (!is_wizard(progr) && accepts == 0) 
	    return make_error_pack(E_NACC);
	
	if (!retrieve(what)
	    ||  (where != NOTHING  &&  !retrieve(where))
	    ||  retrieve(what)->location == where)
	    return no_var_pack();
	
	/* Check to see that we're not trying to violate the hierarchy */
	for (oid = where; oid != NOTHING; oid = retrieve(oid)->location) 
	    if (oid == what)
		return make_error_pack(E_RECMOVE);
	
	/* Remove thing from its old location, if it has one */
	oldloc = retrieve(what)->location;
	if (retrieve(oldloc)) 
	    LL_REMOVE(oldloc, contents, what, next);
	
	/* Move it to the new location, if any */
	if (retrieve(where)) 
	    LL_APPEND(where, contents, what, next);
	retrieve(what)->location = where;
	
	args = make_arglist(what);
	e = call_verb(oldloc, "exitfunc", args, 0);
	/* e != E_TYPE */
	
	if (e == E_NONE) 
	    return make_call_pack(3, data);
	else {
	    free_var(args);
	    if (e == E_MAXREC) 
		return make_error_pack(e);
	}
	/* e == E_INVIND or E_VERBNF, in both cases fall through */

      case 3:			/* Returned from exitfunc call */
	if (retrieve(where) && retrieve(what)
	    && retrieve(what)->location == where) {
	    args = make_arglist(what);
	    e = call_verb(where, "enterfunc", args, 0);
	    /* e != E_TYPE or E_INVIND */
	    
	    if (e == E_NONE) 
		return make_call_pack(4, data);
	    else {
		free_var(args);
		if (e == E_MAXREC) 
		    return make_error_pack(e);
		/* else e == E_VERBNF, fall through */
	    }
	}

      case 4:			/* Returned from enterfunc call */
	return no_var_pack();

      default:
	panic("Unknown PC in DO_MOVE");
	return no_var_pack();	/* Dead code to eliminate compiler warning */
    }
}

static package
bf_move(Var arglist, Byte next, void *vdata, Objid progr)
{
    struct bf_move_data *data = vdata;
    package		p;
    
    if (next == 1) {
	data = alloc_data(sizeof(*data));
	data->what = arglist.v.list[1].v.obj;
	data->where = arglist.v.list[2].v.obj;
    }
    
    p = do_move(arglist, next, data, progr);
    free_var(arglist);
    
    if (p.why != BI_CALL)
	free_data(data);
    
    return p;
}

static void
bf_move_write(void *vdata)
{
    struct bf_move_data *data = vdata;
    db_printf("bf_move data: what = %d, where = %d\n",
	      data->what, data->where);
}

static void *
bf_move_read(FILE *f)
{
    struct bf_move_data *data = alloc_data(sizeof(struct bf_move_data));

    if (fscanf(f, "bf_move data: what = %d, where = %d\n", 
	       &(data->what), &(data->where)) == 2)
	return data;
    else
	return NULL;
}    

void 
register_bf_move(void)
{
  (void)register_function_with_read_write("move", 2, 2, bf_move,
					  bf_move_read, bf_move_write,
					  TYPE_OBJ, TYPE_OBJ);
}


/* $Log: bf_move.c,v $
 * Revision 1.3  1992/08/10  17:28:14  pjames
 * Updated #includes.  Used new make_call_pack.  Used new registration
 * method.  Built in function receives only programmer, not whole
 * Parse_Info.
 *
 * Revision 1.2  1992/07/20  23:51:21  pavel
 * Added rcsid_<filename-root> declaration to hold the RCS ident. string.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
