char rcsid_bf_num[] = "$Id: bf_num.c,v 1.3 1992/08/10 17:36:26 pjames Exp $";

#include "my-time.h"
#include "my-math.h"

#include "functions.h"
#include "random.h"
#include "storage.h"
#include "utils.h"

static package
bf_min(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    int		min = arglist.v.list[1].v.num;
    int		i;
    
    r.type = TYPE_NUM;
    for (i = 2; i <= arglist.v.list[0].v.num; i++) {
	if (arglist.v.list[i].type != TYPE_NUM) {
	    free_var(arglist);
	    return make_error_pack(E_TYPE);
	} else if (arglist.v.list[i].v.num < min)
	    min = arglist.v.list[i].v.num;
    }
    r.v.num = min;
    
    free_var(arglist);
    return make_var_pack(r);
}

static package
bf_max(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    int		max = arglist.v.list[1].v.num;
    int		i;
    
    r.type = TYPE_NUM;
    for (i = 2; i <= arglist.v.list[0].v.num; i++) {
	if (arglist.v.list[i].type != TYPE_NUM) {
	    free_var(arglist);
	    return make_error_pack(E_TYPE);
	} else if (arglist.v.list[i].v.num > max)
	    max = arglist.v.list[i].v.num;
    }
    r.v.num = max;
    
    free_var(arglist);
    return make_var_pack(r);
}

static package
bf_abs(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    r.type = TYPE_NUM;
    if (arglist.v.list[1].v.num < 0)
	r.v.num = -arglist.v.list[1].v.num;
    else
	r.v.num = arglist.v.list[1].v.num;
    free_var(arglist);
    return make_var_pack(r);
}              

static package
bf_sqrt(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;

    if (arglist.v.list[1].v.num < 0) {
	free_var(arglist);
	return make_error_pack(E_INVARG);
    }
    r.type = TYPE_NUM;
    r.v.num = (int)sqrt((double)arglist.v.list[1].v.num);
    free_var(arglist);
    return make_var_pack(r);
}
    
static package
bf_time(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    r.type = TYPE_NUM;
    r.v.num = time(0);
    free_var(arglist);
    return make_var_pack(r);
}

static package
bf_ctime(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    time_t	c;
    char	buffer[50];
    
    if (arglist.v.list[0].v.num == 1) {
	c = arglist.v.list[1].v.num;
    } else {
	c = time(0);
    }
    strftime(buffer, 50, "%a %b %d %H:%M:%S %Y %Z", localtime(&c));
    if (buffer[8] == '0')
	buffer[8] = ' ';
    r.type = TYPE_STR;
    r.v.str = strdup(buffer);
    
    free_var(arglist);
    return make_var_pack(r);
}

static package
bf_random(Var arglist, Byte next, void *vdata, Objid progr)
{
    unsigned num = arglist.v.list[1].v.num;
    
    free_var(arglist);
    
    if (num <= 0) 
	return make_error_pack(E_INVARG);
    else {
	Var r; 
	r.type = TYPE_NUM;
	r.v.num = RANDOM() % num + 1;
	return make_var_pack(r);
    }
}

void
register_bf_num(void)
{ 
    (void) register_function("min", 1, -1, bf_min, TYPE_NUM);
    (void) register_function("max", 1, -1, bf_max, TYPE_NUM);
    (void) register_function("abs", 1, 1, bf_abs, TYPE_NUM);
    (void) register_function("random", 1, 1, bf_random, TYPE_NUM);
    (void) register_function("sqrt", 1, 1, bf_sqrt, TYPE_NUM);
    (void) register_function("time", 0, 0, bf_time);
    (void) register_function("ctime", 0, 1, bf_ctime, TYPE_NUM);
}


/* $Log: bf_num.c,v $
 * Revision 1.3  1992/08/10  17:36:26  pjames
 * Updated #includes.  Used new regisration method.  Add bf_sqrt();
 *
 * Revision 1.2  1992/07/20  23:51:47  pavel
 * Added rcsid_<filename-root> declaration to hold the RCS ident. string.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
