char rcsid_bf_obj[] = "$Id: bf_obj.c,v 1.3 1992/08/10 17:18:58 pjames Exp $";

#include "chparent.h"
#include "execute.h"
#include "functions.h"
#include "list.h"
#include "property.h"
#include "quota.h"
#include "server.h"
#include "storage.h"
#include "structures.h"
#include "utils.h"

/* external */

static package
bf_create(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (parent [, owner]) */
    Objid *data = vdata;
    Var r;  
    
    if (next == 1) { 
	Objid parent, owner;
	
	parent = arglist.v.list[1].v.obj;
	owner = (arglist.v.list[0].v.num == 2
		 ? arglist.v.list[2].v.obj
		 : progr);
	free_var(arglist);
	
	if (!retrieve(parent)
	    || !Obj_perm(progr, parent, FLAG_READ)
	    || (owner != progr && !is_wizard(progr))) 
	    return make_error_pack(E_PERM);

	r.type = TYPE_OBJ;
	r.v.obj = create_object("", parent, owner);
	
	if (r.v.obj == NOTHING)
	    return make_error_pack(E_QUOTA);
	else {
	    enum error e;
	    Var	args;
	    
	    data = alloc_data(sizeof(*data));
	    *data = r.v.obj;
	    args = new_list(0);
	    e = call_verb(r.v.obj, "initialize", args, 0);
	    /* e will not be E_INVIND or E_TYPE */
	    
	    if (e == E_NONE)
		return make_call_pack(2, data);
	    
	    free_data(data);
	    free_var(args);
	    
	    if (e == E_MAXREC) 
		return make_error_pack(e);
	    else /* (e == E_VERBNF) do nothing */
		return make_var_pack(r);
	}
    } else { /* next == 2, returns from initialize verb_call */
	r.type = TYPE_OBJ;
	r.v.obj = *data;
	free_data(data);
	return make_var_pack(r);
    }	  
}

static void
bf_create_write(void *vdata)
{
    db_printf("bf_create data: oid = %d\n", *((Objid *) vdata));
}

static void *
bf_create_read(FILE *f)
{
    Objid *data = alloc_data(sizeof(Objid));
    
    if (fscanf(f, "bf_create data: oid = %d\n", data) == 1)
	return data;
    else
	return NULL;
}

static package
bf_chparent(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (object, new_parent */
    Var r;

    r = chparent(progr, arglist.v.list[1].v.obj, arglist.v.list[2].v.obj);
    free_var(arglist);
    if (r.type == TYPE_ERR) 
	return make_error_pack(r.v.err);
    else 
	return no_var_pack();
}

static package
bf_valid(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (object) */
    Var r;
    
    r.type = TYPE_NUM;
    r.v.num = (retrieve(arglist.v.list[1].v.obj) != 0);
    free_var(arglist);
    return make_var_pack(r);
}

static package
bf_parent(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (object) */
    Var r;
    Objid obj = arglist.v.list[1].v.obj;
    
    free_var(arglist);
    
    if (!retrieve(obj)) 
	return make_error_pack(E_INVARG);
    else {
	r.type = TYPE_OBJ;
	r.v.obj = retrieve(obj)->parent;
	return make_var_pack(r);
    }
}

static package
bf_children(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (object) */
    Var r;
    Objid obj = arglist.v.list[1].v.obj;
    
    free_var(arglist);
    
    if (!retrieve(obj)) 
	return make_error_pack(E_INVARG);
    else {
	Var		child;
	Objid		oid;
	int		i, count = 0;

	for (oid = retrieve(obj)->child;
	     oid != NOTHING;
	     oid = retrieve(oid)->sibling)
	    count++;
	r = new_list(count);
	child.type = TYPE_OBJ;
	for (oid = retrieve(obj)->child, i = 1;
	     oid != NOTHING;
	     oid = retrieve(oid)->sibling, i++) {
	    child.v.obj = oid;
	    r.v.list[i] = child;
	}

	return make_var_pack(r);
    }
}

static package
bf_max_object(Var arglist, Byte next, void *vdata, Objid progr) 
{ /* () */
    Var r;

    r.type = TYPE_OBJ;
    r.v.obj = db_size() - 1;
    free_var(arglist);
    return make_var_pack(r);
}

static enum error 
move_to_nothing(Objid oid, Byte func_pc, Objid *data)
{
    /* All we need to do is the exitfunc and the LL_REMOVE */
    Objid 	oldloc = retrieve(oid)->location;
    Var 	args;
    enum error	e;
    
    if (retrieve(oldloc))
	LL_REMOVE(oldloc, contents, oid, next);
    
    retrieve(oid)->location = NOTHING;
    
    args = new_list(1);
    args.v.list[1].type = TYPE_OBJ;
    args.v.list[1].v.obj = oid;
    
    e = call_verb(oldloc, "exitfunc", args, 0);
    /* e != E_TYPE */
    
    if (e != E_NONE)
	free_var(args);
    
    return e;
}

static package
bf_recycle(Var arglist, Byte func_pc, void *vdata, Objid progr)
{ /* (object) */
    Object     *o;
    Objid       oid;
    Verbdef    *v, *w;
    Propdef    *p, *q;
    int         i, prop_count;
    Var         args;
    enum error	e;
    Objid       *data = vdata;
    
    switch (func_pc) {
      case 1:
	oid = arglist.v.list[1].v.obj;
	free_var(arglist);

	if (!retrieve(oid)) 
	    return make_error_pack(E_INVARG);
	
	if (!is_wizard(progr)  &&  retrieve(oid)->owner != progr)
	    return make_error_pack(E_PERM);
	
	data = alloc_data(sizeof(*data));
	*data = oid;
	args = new_list(0);
	e = call_verb(oid, "recycle", args, 0);
	/* e != E_INVIND or E_TYPE */
	
	if (e == E_NONE) 
	    return make_call_pack(2, data);
	/* else e == E_VERBNF or E_MAXREC; fall through */
	free_var(args);
	goto moving_contents;

      case 2:			/* moving all contents to #-1 */
	free_var(arglist);
	oid = *data;

      moving_contents:
	o = retrieve(oid);
	if (!o) {
	    free_data(data);
	    return no_var_pack();
	}

	while (o->contents != NOTHING) {
	    if (move_to_nothing(o->contents, 2, data) == E_NONE)
		return make_call_pack(2, data);
	}

	if (o->location != NOTHING
	    &&  move_to_nothing(oid, 2, data) == E_NONE)
	    /* Return to same case because this :exitfunc might add new */
	    /* contents to OID or even move OID right back in. */
	    return make_call_pack(2, data);

	/* We can now be confident that OID has no contents and no location */

	/* Do the same thing for the inheritance hierarchy (much easier!) */
	while (o->child != NOTHING)
	    change_parent(o->child, o->parent);
	if (retrieve(o->parent))
	    LL_REMOVE(o->parent, child, oid, sibling);

	/* Finish the demolition. */
	incr_quota(o->owner);
	if (is_player(oid)) {
	    Var		t;
	    
	    t.type = TYPE_OBJ;
	    t.v.obj = oid;
	    players_list = setremove(players_list, t);
	}
	prop_count = count_properties(oid);
	for (i = 0; i < prop_count; i++)
	    free_var(o->propval[i].var);
	if (o->propval)
	    myfree((void *) o->propval, M_PVAL);
	myfree(o->name, M_STRING);
	for (v = o->verbdefs; v; v = w) {
	    if (v->program)
		free_program(v->program);
	    myfree((void *) v->name, M_STRING);
	    w = v->next;
	    myfree((void *) v, M_VERBDEF);
	}
	for (p = o->propdefs; p; p = q) {
	    myfree((void *) p->name, M_STRING);
	    q = p->next;
	    myfree((void *) p, M_PROPDEF);
	} 
	free_object(oid);
	
	free_data(data);
	return no_var_pack();
    }
    
    panic("Can't happen in BF_RECYCLE");
    return no_var_pack();
}

static void
bf_recycle_write(void *vdata)
{
    Objid      *data = vdata;
    
    db_printf("bf_recycle data: oid = %d, cont = 0\n", *data);
}

static void *
bf_recycle_read(FILE *f)
{
    Objid      *data = alloc_data(sizeof(*data));
    int		junk;
    
    if (fscanf(f, "bf_recycle data: oid = %d, cont = %d\n", data, &junk) == 2)
	return data;
    else
	return NULL;
}    

void
    register_bf_obj(void)
{ 
    (void) register_function_with_read_write("create", 1, 2, bf_create,
				      bf_create_read, bf_create_write,
				      TYPE_OBJ, TYPE_OBJ);
    (void) register_function_with_read_write("recycle", 1, 1, bf_recycle,
				      bf_recycle_read, bf_recycle_write,
				      TYPE_OBJ);
    (void) register_function("valid", 1, 1, bf_valid, TYPE_OBJ);
    (void) register_function("parent", 1, 1, bf_parent, TYPE_OBJ);
    (void) register_function("children", 1, 1, bf_children, TYPE_OBJ);
    (void) register_function("chparent", 2, 2, bf_chparent,
			     TYPE_OBJ, TYPE_OBJ);
    (void) register_function("max_object", 0, 0, bf_max_object);
}

/* $Log: bf_obj.c,v $
 * Revision 1.3  1992/08/10  17:18:58  pjames
 * Updated #includes.  Move bf_pass to execute.c Updated to use new
 * registration format.  Built in functions now only receive programmer,
 * instead of entire Parse_Info.
 *
 * Revision 1.2  1992/07/20  23:52:48  pavel
 * Added rcsid_<filename-root> declaration to hold the RCS ident. string.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
