char rcsid_bf_type[] = "$Id: bf_type.c,v 1.3 1992/08/10 17:15:58 pjames Exp $";

#include "functions.h"
#include "my-stdlib.h"
#include "streams.h"
#include "storage.h"
#include "unparse.h"
#include "utils.h"

char *
list2str(Var *args)
{
    static Stream      *str = 0;
    int			i;
    
    if (!str)
	str = new_stream(100);
    
    for (i = 1; i <= args[0].v.num; i++) {
	switch (args[i].type) {
	  case TYPE_NUM:
	    stream_printf(str, "%d", args[i].v.num);
	    break;
	  case TYPE_OBJ:
	    stream_printf(str, "#%d", args[i].v.obj);
	    break;
	  case TYPE_STR:
	    stream_add_string(str, args[i].v.str);
	    break;
	  case TYPE_ERR:
	    stream_add_string(str, unparse_error(args[i].v.err));
	    break;
	  case TYPE_LIST:
	    stream_add_string(str, "{list}");
	    break;
	}
    }
    
    return strdup(reset_stream(str));
}

static int
parse_number(const char *str, int *result)
{
    char	*p;
    int		negative = 0;
    
    while (*str  &&  *str == ' ')
	str++;
    if (*str == '-') {
	str++;
	negative = 1;
    }
    *result = strtol(str, &p, 10);
    if (p == str)
	return 0;
    while (*p) {
	if (*p != ' ')
	    return 0;
	p++;
    }
    if (negative)
	*result = -*result;
    return 1;
}


static int
parse_object(const char *str, Objid *result)
{
    int		number;
    
    while (*str  &&  *str == ' ')
	str++;
    if (*str == '#')
	str++;
    if (parse_number(str, &number)) {
	*result = number;
	return 1;
    } else
	return 0;
}

static enum error
become_number(Var in, int *ret, char called_from_tonum)
{ 
    switch (in.type) {
      case TYPE_NUM:
	*ret = in.v.num;
	break;
      case TYPE_STR: 
	if (!(called_from_tonum
	      ? parse_number(in.v.str, ret)
	      : parse_object(in.v.str, ret)))
#if I_HAD_MY_DRUTHERS
	    return E_INVARG;
#else
	*ret = 0;
#endif
	break;
      case TYPE_OBJ:
	*ret = in.v.obj;
	break;
      case TYPE_ERR:
	*ret = in.v.err;
	break;
      case TYPE_LIST:
	return E_TYPE;
    }
    return E_NONE;
}

static package
bf_tonum(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r; 
    enum error e;
    
    r.type = TYPE_NUM;
    e = become_number(arglist.v.list[1], &(r.v.num), 1);
    
    free_var(arglist);
    if (e != E_NONE)
	return make_error_pack(e);
    
    return make_var_pack(r);
}

static package
bf_toobj(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r; 
    enum error e;
    
    r.type = TYPE_OBJ;
    e = become_number(arglist.v.list[1], &(r.v.obj), 0);
    
    free_var(arglist);
    if (e != E_NONE)
	return make_error_pack(e);
    
    return make_var_pack(r);
}

static package
bf_typeof(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    r.type = TYPE_NUM;
    r.v.num = (int) arglist.v.list[1].type;
    free_var(arglist);
    return make_var_pack(r);
}

static package
bf_tostr(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    r.type = TYPE_STR;
    r.v.str = list2str(arglist.v.list);
    free_var(arglist);
    return make_var_pack(r);
}

void
register_bf_type(void)
{ 
    (void) register_function("typeof", 1, 1, bf_typeof, TYPE_ANY);
    (void) register_function("tostr", 0, -1, bf_tostr);
    (void) register_function("tonum", 1, 1, bf_tonum, TYPE_ANY);
    (void) register_function("toobj", 1, 1, bf_toobj, TYPE_ANY);
}


/* $Log: bf_type.c,v $
 * Revision 1.3  1992/08/10  17:15:58  pjames
 * Updated #includes.  Updated to use new registration format.  Built in
 * functions now only receive programmer, instead of entire Parse_Info.
 *
 * Revision 1.2  1992/07/20  23:55:02  pavel
 * Added rcsid_<filename-root> declaration to hold the RCS ident. string.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
