char rcsid_chparent[] = "$Id: chparent.c,v 1.3 1992/08/10 16:53:30 pjames Exp $";

#include "match.h"
#include "chparent.h"
#include "property.h"
#include "storage.h"
#include "structures.h"
#include "unparse.h"
#include "utils.h"

static int
check_props(Objid parent, Objid thing)
{
    /* Return true iff PARENT defines no properties that are also defined by */
    /* either THING or any of THING's descendants. */

    Propid	junk;
    Objid	c;
    Propdef    *p;

    for (p = retrieve(thing)->propdefs; p; p = p->next)
	if (match_property(p->name, parent, &junk))
	    return 0;

    for (c = retrieve(thing)->child; c != NOTHING; c = retrieve(c)->sibling)
	if (!check_props(parent, c))
	    return 0;

    return 1;
}


static void
fix_props(Objid oid, int parent_local, int old, int new, int common)
{
    Propdef    *p;
    Object     *me = retrieve(oid);
    Object     *parent = retrieve(me->parent);
    Pval       *new_pvals;
    int		local = parent_local;
    int		i;
    Objid	c;

    for (p = me->propdefs; p; p = p->next)
	local++;

    for (i = local; i < local + old; i++)
	free_var(me->propval[i].var);

    if (local + common + new != 0) {
	new_pvals = (Pval *) mymalloc((local + new + common) * sizeof(Pval),
				      M_PVAL);
	for (i = 0; i < local; i++)
	    new_pvals[i] = me->propval[i];
	for (i = 0; i < new; i++) {
	    Pval	pv;

	    pv = parent->propval[parent_local + i];
	    new_pvals[local + i] = pv;
	    new_pvals[local + i].var = vardup(pv.var);
	    if (pv.perms & (1 << PP_CHOWN))
		new_pvals[local + i].owner = me->owner;
	}
	for (i = 0; i < common; i++)
	    new_pvals[local + new + i] = me->propval[local + old + i];
    } else
	new_pvals = 0;

    if (me->propval)
	myfree((void *) me->propval, M_PVAL);
    me->propval = new_pvals;

    for (c = me->child; c != NOTHING; c = retrieve(c)->sibling)
	fix_props(c, local, old, new, common);
}

Var
change_parent(Objid thing, Objid parent)
{
    Objid	oid, tid, common, old_parent;
    Object	*o = retrieve(thing);
    int		new_props, common_props, old_props;
    Var		r;

    /*
     * check for class hierarchy violations
     */
    for (oid = parent; oid != NOTHING; oid = retrieve(oid)->parent) {
	if (oid == thing) {
	    r.type = TYPE_ERR;
	    r.v.err = E_RECMOVE;
	    return r;
	}
    }

    /*
     * check for a common ancestor between old & new parent
     */
    common = NOTHING;
    for (oid = thing; oid != NOTHING; oid = retrieve(oid)->parent) {
	for (tid = parent; tid != NOTHING; tid = retrieve(tid)->parent) {
	    if (oid == tid) {
		common = oid;
		goto endouter;
	    }
	}
    }
    endouter:

    if (common != NOTHING)
	common_props = count_properties(common);
    else
	common_props = 0;

    old_parent = o->parent;
    old_props = count_properties(old_parent) - common_props;
    new_props = count_properties(parent) - common_props;

    if (old_parent != NOTHING) {
	LL_REMOVE(old_parent, child, thing, sibling)
    }
    if (parent != NOTHING) {
	LL_APPEND(parent, child, thing, sibling)
    }
    o->parent = parent;

    fix_props(thing, 0, old_props, new_props, common_props);

    r.type = TYPE_NUM;
    r.v.num = 0;
    return r;
}

Var
chparent(Objid player, Objid thing, Objid parent)
{
    Var		r;

    if (!retrieve(thing)
	|| (retrieve(thing)->owner != player && !is_wizard(player))
	|| (!retrieve(parent) && parent != NOTHING)
	|| (retrieve(parent) && !Obj_perm(player, parent, FLAG_READ))) {
	r.type = TYPE_ERR;
	r.v.err = E_PERM;
	return r;
    } else if (!check_props(parent, thing)) {
	r.type = TYPE_ERR;
	r.v.err = E_INVARG;
	return r;
    } else
	return change_parent(thing, parent);
}

/* $Log: chparent.c,v $
 * Revision 1.3  1992/08/10  16:53:30  pjames
 * Updated #includes.
 *
 * Revision 1.2  1992/07/21  00:00:58  pavel
 * Added rcsid_<filename-root> declaration to hold the RCS ident. string.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
