char rcsid_eval_vm[] = "$Id: eval_vm.c,v 1.1 1992/08/10 16:20:00 pjames Exp $";

#include "eval_vm.h"
#include "execute.h"
#include "log.h"
#include "storage.h"
#include "structures.h"
#include "tasks.h"
#include "unparse.h"

/**** external functions ****/

void
free_vm(vm the_vm, int stack_too)
{
    int i;

    if (stack_too) 
	for (i = the_vm->top_activ_stack; i >= 0; i--)
	    free_activation(the_vm->activ_stack[i], 1);
    myfree((void *) the_vm, M_VM);
}

Var
resume_suspended_task(vm the_vm)
{
    set_task_kind(TASK_SUSPENDED);
    return resume_from_previous_vm(the_vm, NULL);
}

Var 
resume_reading_vm(vm the_vm, char *input)
{
    set_task_kind(TASK_INPUT);
    return resume_from_previous_vm(the_vm, input);
}


Objid 
inputting_player(vm the_vm)
{
    return the_vm->activ_stack[0].player;
}

activation
top_activ(vm the_vm)
{
    return the_vm->activ_stack[the_vm->top_activ_stack];
}

Objid
progr_of_cur_verb(vm the_vm) 
{
    return top_activ(the_vm).progr;
}

unsigned
suspended_lineno_of_vm(vm the_vm)
{
    activation top;

    top = top_activ(the_vm);
    return get_lineno(top.prog, top.pc - 2, /* pc -2 to points back to 
					 OP_BI_FUNC_CALL(suspend); this 
					 is unimportant but a good practice */
		      (the_vm->top_activ_stack == 0) 
		      ? the_vm->root_activ_vector 
		      : MAIN_VECTOR);
}

/**** read/write data base ****/

void 
write_vm(vm the_vm)
{
    unsigned i;

    db_printf("%d %d %d\n", the_vm->top_activ_stack, 
	      the_vm->root_activ_vector, the_vm->func_id);
    
    for (i = 0; i <= the_vm->top_activ_stack; i++)
	write_activ(the_vm->activ_stack[i]);
}

char
read_vm(FILE *f, vm *addr_for_the_vm)
{
    unsigned i, j;

    vm the_vm = mymalloc(sizeof(vmstruct), M_VM);

    i = fscanf(f, "%d %d %d\n", &(the_vm->top_activ_stack), 
	       &(the_vm->root_activ_vector), &(the_vm->func_id));
    if (i != 3) {
	errlog("READ_VM: Bad vm header\n");
	myfree((void *) the_vm, M_VM);
	return -1;
    }

    for (i = 0; i <= the_vm->top_activ_stack; i++)
	if (read_activ(f, &(the_vm->activ_stack[i]),
		       i == 0 ? the_vm->root_activ_vector : MAIN_VECTOR) < 0) {
	    errlog("READ_VM: Bad activ number %d\n", i);

	    for (j = 0; j < i; j++)
		free_activation(the_vm->activ_stack[i], 1 /* data_too */);
	    return -1;
	}

    *addr_for_the_vm = the_vm;    
    return 0;
}

/* $Log: eval_vm.c,v $
 * Revision 1.1  1992/08/10  16:20:00  pjames
 * Initial RCS-controlled version
 *
   */
