#ifndef Structures_h
#define Structures_h 1

#include "my-stdio.h"

typedef struct Names Names;
typedef struct Env Env;
typedef struct Stmt Stmt;
typedef struct Expr Expr;
typedef struct Arg_List Arg_List;
typedef struct Cond_Arm Cond_Arm;

typedef int  Objid;
typedef int  Prepid;
typedef int  Propid;

enum flag {
    FLAG_PLAYER, FLAG_PROGRAMMER, FLAG_WIZARD, FLAG_OBSOLETE_1,
    FLAG_READ, FLAG_WRITE, FLAG_OBSOLETE_2
};

enum vperms { VP_READ, VP_WRITE, VP_EXEC, VP_DEBUG };
enum vopt { OPT_NONE, OPT_ANY, OPT_THIS, OPT_FAIL };

#define VP_DOBJSHIFT  4
#define VP_IOBJSHIFT  6
#define VP_OBJMASK    0x3
#define VP_PERMMASK   0xF

enum propperms { PP_READ, PP_WRITE, PP_CHOWN };

#define Prop_perm(player, prop, which)	\
    (player == prop->owner || is_wizard(player) || (prop->perms & (1<<which)))
#define Verb_perm(player, verb, which)	\
    (player == verb->owner || is_wizard(player) || (verb->perms & (1<<which)))
#define Obj_perm(player, oid, which)	\
    (player == retrieve(oid)->owner	\
     || is_wizard(player)		\
     || (retrieve(oid)->flags & (1<<which)))
/*
 * Special Objid's
 */
#define NOTHING		-1
#define AMBIGUOUS	-2
#define FAILED_MATCH	-3

enum error { E_NONE, E_TYPE, E_DIV, E_PERM, E_PROPNF, E_VERBNF,
       E_VARNF, E_INVIND, E_RECMOVE, E_MAXREC,
       E_RANGE, E_ARGS, E_NACC, E_INVARG, E_QUOTA };

/* Pseudo-properties */
#define LOCATION	-100
#define OWNER		-101
#define NAME		-102
#define CONTENTS	-103
#define FLAG		-200

#define Arraysize(x,y) (sizeof(x) / sizeof(y))

#define LL_REMOVE(where, listname, what, nextname) { \
    Objid lid; \
    if (retrieve(where)->listname == what) \
	retrieve(where)->listname = retrieve(what)->nextname; \
    else { \
	for (lid = retrieve(where)->listname; lid != NOTHING; \
	      lid = retrieve(lid)->nextname) { \
	    if (retrieve(lid)->nextname == what) { \
		retrieve(lid)->nextname = retrieve(what)->nextname; \
		break; \
	    } \
	} \
    } \
    retrieve(what)->nextname = NOTHING; \
}

#define LL_APPEND(where, listname, what, nextname) { \
    Objid lid; \
    if (retrieve(where)->listname == NOTHING) { \
	retrieve(where)->listname = what; \
    } else { \
	for (lid = retrieve(where)->listname; \
	     retrieve(lid)->nextname != NOTHING; \
	     lid = retrieve(lid)->nextname) \
	    ; \
	retrieve(lid)->nextname = what; \
    } \
    retrieve(what)->nextname = NOTHING; \
}

typedef union Value  Value;
typedef struct Var  Var;

union Value {
    char	*str;
    int		num;
    Objid	obj;
    enum error	err;
    Var		*list;
};

typedef enum var_type {
    TYPE_NUM, TYPE_OBJ, TYPE_STR, TYPE_ERR, TYPE_LIST
    } var_type;
#define TYPE_ANY ((var_type) -1)

struct Var {
    Value	v;
    var_type	type;
};

#include "bytecodes.h"
typedef struct Program Program;

struct Program {
  unsigned     first_lineno;
  unsigned     ref_count;

  Bytecodes    main_vector;

  unsigned     num_literals;
  Var         *literals;

  unsigned     fork_vectors_size;
  Bytecodes   *fork_vectors;
  
  unsigned     num_var_names;
  char       **var_names;
};

#define ADD_REF_TO_PROGRAM(p)   p->ref_count++
void free_program(Program *p);

/*** end of ak's changes ***/

typedef	struct Verbdef  Verbdef;

struct Verbdef {
    char       *name;
    Program    *program;
    Objid	owner;
    short	perms;
    short	prep;
    Verbdef    *next;
};

typedef struct Propdef  Propdef;

struct Propdef {
    char	*name;
    Propdef	*next;
};

typedef struct Pval {
    Var		var;
    Objid	owner;
    short	perms;
} Pval;

typedef struct Object {
    Objid	id;
    Objid	owner;
    Objid	location;
    Objid	contents;
    Objid	next;

    Objid	parent;
    Objid	child;
    Objid	sibling;

    char	*name;
    int		flags;

    Verbdef	*verbdefs;
    Propdef	*propdefs;
    Pval	*propval;
} Object;

typedef struct parseinfo	Parse_Info;

struct parseinfo {
    Parse_Info *next;

    Var		args;	/* arguments to the verb */

    Objid	this;	/* what object has inherited the verb */
    Objid	dobj;	/* direct object */
    Objid	iobj;	/* indirect object */
    Objid	player;	/* what player caused the verb to execute */
    Objid	caller; /* what object is calling this verb */
    Objid	progr;	/* player who programmed verb (verbdef.owner) */
    Objid	vloc;	/* where verbdef is located */

    char       *argstr;		/* arguments to verb */
    char       *dobjstr;	/* direct object typed by player */
    char       *iobjstr;	/* indirect object string */
    char       *prepstr;	/* preposition string */
    char       *verb;		/* verb (as typed by player) */
    char       *verbname;	/* name of verb (from verbdef) */

    int		prep;	/* number of preposition */

    int		debug;	/* debug flag for verbdef */
};

#endif /* !Structures_h */

/* $Log: structures.h,v $
 * Revision 1.6  1992/08/14  00:00:36  pavel
 * Converted to a typedef of `var_type' = `enum var_type'.
 *
 * Revision 1.5  1992/08/10  16:52:00  pjames
 * Moved several types/procedure declarations to storage.h
 *
 * Revision 1.4  1992/07/30  21:24:31  pjames
 * Added M_COND_ARM_STACK and M_STMT_STACK for vector.c
 *
 * Revision 1.3  1992/07/28  17:18:48  pjames
 * Added M_COND_ARM_STACK for unparse.c
 *
 * Revision 1.2  1992/07/27  18:21:34  pjames
 * Changed name of ct_env to var_names, const_env to literals and
 * f_vectors to fork_vectors, removed M_CT_ENV, M_LOCAL_ENV, and
 * M_LABEL_MAPS, changed M_CONST_ENV to M_LITERALS, M_IM_STACK to
 * M_EXPR_STACK, M_F_VECTORS to M_FORK_VECTORS, M_ID_LIST to M_VL_LIST
 * and M_ID_VALUE to M_VL_VALUE.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
